% Matlab script to find implement the discrete time filter,
% and calculate the PSD of the output. It plots the output.
% It runs psdfit first to find the filter coefficients
%
global RPOLES CPOLESre CPOLESim RRES CRESre CRESim X Xr Xi;

psdfit;

period = 80e-9;
Nskip = 2^3;
Npsd = 2^16 ;
Nrun = 2^18;


F = logspace (2, 7.5, 200);
[H, Szz] = psdzac(F*period, z_rpoles, z_cpoles, rres, cres);
Szz = gain^2 * Szz;

figure(10);
loglog(F, Szz, '--' );

RPOLES = z_rpoles
CPOLESre = real(z_cpoles)
CPOLESim = imag(z_cpoles)
RRES = rres
CRESre = real(cres)
CRESim = imag(cres)

X = RPOLES-RPOLES;
Xr = CPOLESre-CPOLESre;
Xi = Xr;

fid = fopen('noise.out', 'w');
% fidx = fopen('X.out', 'w');

gain = gain/sqrt(2*period);
for i=1:Nrun,
  if (mod(i,1000)==0) i
  X
  Xr
  Xi
  end;
  in = randn;
  y = prfilter(in);
  out = gain * y;
  if (i>Nskip) fprintf(fid, '%g\t%g\n', (i-Nskip)*period, out); end;
%  fprintf(fidx, '%g\t', i*period, X, Xr, Xi); fprintf(fidx, '\n');
end;

fclose(fid);
% fclose(fidx);

load noise.out;

noi = noise(:,2);
time = noise(:,1);
[Std,Ftd] = psd(noi, Npsd, 1/period, Npsd);
Std = Std * 2 * period;
loglog(Ftd, Std, '.', F, Szz, '-');

% process results
N=100;
c=1e-1;
Nl = length(Ftd);
f1 = Ftd(1);
f2 = Ftd(Nl);
for i=1:N,
  li = round(Nl^(i/N)); 
  win = round(c * li);
  up = min (Nl, li+win);
  SmoothFtd(i) = mean(Ftd(li-win:up));
  SmoothStd(i) = mean(Std(li-win:up));
end;

%loglog( F, Szz, '-', Ftd, Std, 'r--', SmoothFtd, SmoothStd, 'm.');
loglog( F, Szz, '-', SmoothFtd, SmoothStd, 'mo');
xlabel('Frequency (Hz)');
ylabel('Input Referred Noise (V^2/Hz)');
print -depsc 'psdfilter.eps'
grid;

